import 'dart:io';
import 'dart:typed_data';
import 'package:flutter/cupertino.dart';
import 'package:nui_core/data/nui_ent.dart';
import 'package:nui_database/models/nui_database_entity.dart';
import 'package:nui_database/models/nui_database_entity_field.dart';
import 'package:nui_database/models/nui_database_entity_mapper.dart';

class NUIMediaFile extends NUIDBEntity<NUIMediaFile>{
  static const String MEDIA_TYPE_VIDEO = "video";
  static const String MEDIA_TYPE_IMAGE = "photo";

  NUIMediaFile.empty();
  NUIMediaFile({@required this.path, @required this.width, @required this.height, this.type: MEDIA_TYPE_IMAGE, this.videoPath, this.description, this.thumbData, this.createdDate, this.isLocal:false});

  String path;
  String videoPath;
  int width;
  int height;
  String type;
  String description;

  Uint8List cropped;
  String croppedFileName;
  String originalPath;
  Future<Uint8List> thumbData;
  DateTime createdDate;

  //Other attributes
  bool uploaded = false;
  bool isLocal = false;
  bool isSquare = false;
  bool centerCrop = true;
  int sequence;
  bool adjustByWidth = false;
  bool adjustByRatio = false;
  double maxHeightToWidthRatio;
  double minHeightToWidthRatio;

  //Video attributes
  int videoProgress = 0;
  int videoDuration;
  bool isFinished = false;
  bool isMute = true;
  bool loop = false;

  @override
  NUIDBEntityMapper<NUIMediaFile> mapper(){
    return NUIMediaFileEntity();
  }

  bool isImage(){
    return type == MEDIA_TYPE_IMAGE;
  }

  bool isVideo(){
    return type == MEDIA_TYPE_VIDEO;
  }

  File getFile(){
    return File(path);
  }

  bool hasSize(){
    return height != null && width != null && height > 0 && width > 0;
  }

  bool isLandscape(){
    if(!hasSize()) return false;
    return width > height;
  }

  bool isPortrait(){
    if(!hasSize()) return false;
    return height > width;
  }

  double heightToWidthRatio(){
    if(!hasSize()) return 0;
    return height.toDouble()/width.toDouble();
  }
  double widthToHeightRatio(){
    if(!hasSize()) return 0;
    return width.toDouble()/height.toDouble();
  }

  double processHeightToWidthRatio(){
    if(!hasSize()) return 0;
    final ratio = heightToWidthRatio();
    if(centerCrop || adjustByRatio){
      if(maxHeightToWidthRatio != null && ratio > maxHeightToWidthRatio) {
        return maxHeightToWidthRatio;
      }
      else if(minHeightToWidthRatio != null && ratio < minHeightToWidthRatio){
        return minHeightToWidthRatio;
      }
    }
    if(isSquare == true){
      return 1;
    }
    return ratio;
  }
}

class NUIMediaFileEntity extends NUIDBEntityMapper<NUIMediaFile>{
  @override
  String entityName(){
    return "NUIMediaFile";
  }

  @override
  NUIMediaFile newInstance(){
    return NUIMediaFile.empty();
  }

  @override
  List<NUIDBField> dbFields(){
    return [
      NUIDBField<NUIMediaFile>(name: "path", type: NUIEntType.STRING, setter: (file, value) => file.path = value, getter: (data) => data.path, primaryKey: true),
      NUIDBField<NUIMediaFile>(name: "videoPath", type: NUIEntType.STRING, setter: (file, value) => file.videoPath = value, getter: (data) => data.videoPath),
      NUIDBField<NUIMediaFile>(name: "width", type: NUIEntType.INTEGER, setter: (file, value) => file.width = value, getter: (data) => data.width),
      NUIDBField<NUIMediaFile>(name: "height", type: NUIEntType.INTEGER, setter: (file, value) => file.height = value, getter: (data) => data.height),
      NUIDBField<NUIMediaFile>(name: "type", type: NUIEntType.STRING, setter: (file, value) => file.type = value, getter: (data) => data.type),
      NUIDBField<NUIMediaFile>(name: "description", type: NUIEntType.STRING, setter: (file, value) => file.description = value ?? null, getter: (data) => data.description),
      NUIDBField<NUIMediaFile>(name: "sequence", type: NUIEntType.INTEGER, setter: (file, value) => file.sequence = value ?? null, getter: (data) => data.sequence),
      NUIDBField<NUIMediaFile>(name: "uploaded", type: NUIEntType.BOOLEAN, setter: (file, value) => file.uploaded = value ?? false, getter: (data) => data.uploaded),
      NUIDBField<NUIMediaFile>(name: "isLocal", type: NUIEntType.BOOLEAN, setter: (file, value) => file.isLocal = value ?? false, getter: (data) => data.isLocal)
    ];
  }
}