import 'dart:typed_data';
import 'dart:ui';
import 'package:crop/crop.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:image_gallery_saver/image_gallery_saver.dart';
import 'package:line_awesome_flutter/line_awesome_flutter.dart';
import 'package:nui_animation/nui_animation.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_media/model/nui_media_file.dart';
import 'package:nui_media/nui_media.dart';

Future<NUIMediaFile> showNUIMediaCropping(BuildContext context, {NUIMediaFile mediaFile, bool saveAsFile: false, BoxShape cropShape: BoxShape.rectangle, bool enableRotate: true, double ratio: 1, Color accentColor, Color accentColorBright}) async{
  final result = await showNUIDialogForResult<NUIMediaFile>(
      context: context,
      content: (context) => NUIMediaCropView(
        accentColor: accentColor,
        accentColorBright: accentColorBright,
        mediaFile: mediaFile,
        enableRotate: enableRotate,
        cropShape: cropShape,
        saveAsFile: saveAsFile,
        ratio: ratio,
      ),
      opacity: 0.7,
      transition: PageTransition.BLUR_BACKGROUND
  );
  return result;
}

class NUIMediaCropView extends StatefulWidget {
  NUIMediaCropView({this.enableRotate: true, this.ratio: 1, this.mediaFile, this.saveAsFile: false, this.cropShape: BoxShape.rectangle, this.accentColor, this.accentColorBright});
  final bool enableRotate;
  final double ratio;
  final Color accentColor;
  final Color accentColorBright;
  final NUIMediaFile mediaFile;
  final bool saveAsFile;
  final BoxShape cropShape;
  @override
  _NUIMediaCropViewState createState() => _NUIMediaCropViewState();
}

class _NUIMediaCropViewState extends State<NUIMediaCropView> {
  CropController controller;
  double _rotation = 0;
  BoxShape shape = BoxShape.rectangle;
  Uint8List _croppedImage;
  bool _cropping = false;
  bool _saving = false;

  @override
  void initState(){
    super.initState();
    controller = CropController(aspectRatio: widget.ratio);
  }

  void crop(BuildContext context) async{
    setState(() {
      _cropping = true;
    });

    final pixelRatio = MediaQuery.of(context).devicePixelRatio;
    final cropped = await controller.crop(pixelRatio: pixelRatio);
    var byteData = await cropped.toByteData(format: ImageByteFormat.png);
    _croppedImage = byteData.buffer.asUint8List();
    logNUI("NUIMediaCropView", "Cropped image obtained");

    setState(() {
      _cropping = false;
    });
  }

  void confirm(BuildContext context) async{
    if(_croppedImage == null) return;
    if(widget.saveAsFile == true){
      //Save to phone storage

      setState(() {
        _saving = true;
      });

      final targetName = "CROPPED" + dateToString(DateTime.now(), format: NUIDTUtil.FORMAT_yyyy_MM_dd).replaceAll("-", "")  + "_" + NUIMediaUtil.nameFromPath(widget.mediaFile.path, withExtension: false);
      final result = await ImageGallerySaver.saveImage(_croppedImage, name: targetName);

      await NUIAsync.delayForResult(Duration(seconds: 3), () => null);
      logNUI("NUIMediaCropView", "Cropped file result : $result");

      setState(() {
        _saving = false;
      });

      widget.mediaFile.cropped = _croppedImage;
      widget.mediaFile.originalPath = widget.mediaFile.path;
      widget.mediaFile.croppedFileName = targetName;
      Navigator.pop(context, widget.mediaFile);
    }
    else{
      //Save only the bytes
      widget.mediaFile.cropped = _croppedImage;
      widget.mediaFile.originalPath = widget.mediaFile.path;
      Navigator.pop(context, widget.mediaFile);
    }
  }

  void _retry(){
    setState(() {
      _croppedImage = null;
    });
  }

  void _back(){
    Navigator.pop(context, null);
  }

  @override
  Widget build(BuildContext context) {
    final isLocalImage = widget.mediaFile.isLocal == true;

    return Scaffold(
      backgroundColor: NUIColors.NUITransparent,
      body: NUIScreen(
        screenCode: "NUIMediaCropView",
        overlayStyle: NUIOverlayStyle(darkStatusBar: false),
        content: (context) => Container(
          alignment: Alignment.center,
          width: double.infinity,
          height: double.infinity,
          color: NUIColors.NUIBlack.withOpacity(0.7),
          padding: EdgeInsets.only(top: 10),
          child: NUIColumn(
            mainAxisSize: MainAxisSize.min,
            children: [
              Container(
                  height: screenHeight(context) * 0.55,
                  child: Stack(
                    alignment: Alignment.center,
                    children: [
                      Visibility(
                        visible: _croppedImage == null,
                        child: Center(
                          child: Crop(
                            controller: controller,
                            shape: widget.cropShape,
                            backgroundColor: NUIColors.NUIBlack.withOpacity(0.3),
                            dimColor: NUIColors.NUIBlack.withOpacity(0.6),
                            background: NUIClipView(
                              borderRadius: BorderRadius.circular(0),
                              child: Stack(
                                children: [
                                  Container(
                                      width: double.infinity,
                                      height: screenHeight(context) * 0.55,
                                      child: isLocalImage
                                          ? Image.file(widget.mediaFile.getFile(), fit: BoxFit.cover)
                                          : loadImage(widget.mediaFile.path, fit: BoxFit.cover)
                                  ),
                                  BackdropFilter(
                                    filter: ImageFilter.blur(sigmaX: 5.0, sigmaY: 5.0),
                                    child: Container(
                                      height: screenHeight(context) * 0.55,
                                      width: double.infinity,
                                      color: NUIColors.NUIBlack.withOpacity(0.3),
                                    ),
                                  ),
                                ],
                              ),
                            ),
                            interactive: _croppedImage == null,
                            child: isLocalImage
                                ? Image.file(widget.mediaFile.getFile())
                                : loadImage(widget.mediaFile.path, fit: BoxFit.cover)
                          )
                        ),
                      ),
                      Visibility(
                        visible: _croppedImage != null,
                        child: Center(
                          child: _croppedImage != null ? NUIZoomOverlay(twoTouchOnly: true, child: Image.memory(_croppedImage)) : SizedBox(width: 10, height: 10,),
                        ),
                      ),
                      Visibility(
                        visible: widget.ratio > 1 ? true : false,
                        child: Container(
                          height: screenHeight(context) * 0.55,
                          width: double.infinity,
                          child: NUIColumn(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: [
                              Container(
                                width: double.infinity,
                                height: 10,
                                decoration: NUIBoxDecoration(
                                  gradient: NUILinearGradient(
                                    colors: [
                                      NUIColors.NUIBlack.withOpacity(0.8),
                                      NUIColors.NUIBlack.withOpacity(0.0)
                                    ],
                                    orientation: GradientOrientation.T_B
                                  )
                                ),
                              ),
                              Container(
                                width: double.infinity,
                                height: 10,
                                decoration: NUIBoxDecoration(
                                    gradient: NUILinearGradient(
                                        colors: [
                                          NUIColors.NUIBlack,
                                          NUIColors.NUIBlack.withOpacity(0.0)
                                        ],
                                        orientation: GradientOrientation.B_T
                                    )
                                ),
                              )
                            ],
                          ),
                        ),
                      ),
                      Visibility(
                        visible: _cropping || _saving,
                        child: Positioned(
                          top: 7,
                          right: 7,
                          child: Container(
                            height: 40,
                            width: 40,
                            alignment: Alignment.center,
                            child:  SizedBox(width: 24, height: 24, child: CircularProgressIndicator(valueColor: AlwaysStoppedAnimation<Color>(NUIColors.NUIWhite), strokeWidth: 1.5, backgroundColor: NUIColors.NUIWhite.withOpacity(0.5))),
                          ),
                        ),
                      ),
                    ],
                  )
              ),
              Padding(
                padding: const EdgeInsets.only(left: 15, right: 15, top: 15, bottom: 15),
                child: NUIRow(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  mainAxisSize: MainAxisSize.max,
                  children: [
                    NUIBouncingAnimation(
                      child: Container(
                          height: 40,
                          width: 40,
                          alignment: Alignment.center,
                          decoration: NUIOutlineDecoration(
                              outlineColor: NUIColors.NUIWhite.withOpacity(0.6),
                              gradient: NUILinearGradient(
                                  colors: [
                                    NUIColors.NUIWhite.withOpacity(0.5),
                                    NUIColors.NUIWhite.withOpacity(0.3),
                                  ],
                                  orientation: GradientOrientation.TL_BR
                              ),
                              thickness: 1.5,
                              borderRadius: BorderRadius.circular(20)
                          ),
                          child: Icon(_croppedImage == null ? LineAwesomeIcons.arrow_left : LineAwesomeIcons.undo, color: NUIColors.NUIWhite, size: 24)
                      ),
                      onTap: (){
                        if(_croppedImage == null) {
                          _back();
                        }
                        else{
                          _retry();
                        }
                      },
                      scaleFactor: 0.5,
                    ),
                    NUIBouncingAnimation(
                      child: Container(
                          height: 40,
                          width: 40,
                          alignment: Alignment.center,
                          decoration: NUIOutlineDecoration(
                              outlineColor: NUIColors.NUIWhite.withOpacity(0.6),
                              gradient: NUILinearGradient(
                                  colors: [
                                    NUIColors.NUIWhite.withOpacity(0.5),
                                    NUIColors.NUIWhite.withOpacity(0.3),
                                  ],
                                  orientation: GradientOrientation.TL_BR
                              ),
                              thickness: 1.5,
                              borderRadius: BorderRadius.circular(20)
                          ),
                          child: Icon(_croppedImage == null ? LineAwesomeIcons.crop : LineAwesomeIcons.check, color: NUIColors.NUIWhite, size: 24)
                      ),
                      onTap: (){
                        if(_croppedImage == null) {
                          crop(context);
                        }
                        else{
                          confirm(context);
                        }
                      },
                      scaleFactor: 0.5,
                    ),
                  ],
                ),
              )
            ],
          ),
        ),
      ),
    );
  }
}

